@@ -534,7 +534,7 @@ namespace dlib
                 item.add(d,r);
             }
         }
-        catch (serialization_error e)
+        catch (serialization_error& e)
         { 
             item.clear();
             throw serialization_error(e.info + "\n   while deserializing object of type binary_search_tree_kernel_1"); 
