@@ -67,7 +67,7 @@ const column_vector rosen_derivative (const column_vector& m)
     return res;
 }
 
-// This function computes the Hessian matrix for the rosen() fuction.  This is
+// This function computes the Hessian matrix for the rosen() function.  This is
 // the matrix of second derivatives.
 matrix<double> rosen_hessian (const column_vector& m)
 {
@@ -304,7 +304,8 @@ int main() try
     auto result = find_min_global(complex_holder_table, 
                                   {-10,-10}, // lower bounds
                                   {10,10}, // upper bounds
-                                  max_function_calls(300));
+                                  std::chrono::milliseconds(500) // run this long
+                                  );
 
     cout.precision(9);
     // These cout statements will show that find_min_global() found the
