@@ -1,16 +1,14 @@
 // Copyright (C) 2013  Davis E. King (davis@dlib.net)
 // License: Boost Software License   See LICENSE.txt for the full license.
 
+#include <string>
+#include "opaque_types.h"
 #include <dlib/python.h>
-#include <boost/shared_ptr.hpp>
 #include <dlib/matrix.h>
 #include <dlib/svm_threaded.h>
-#include <boost/python/suite/indexing/vector_indexing_suite.hpp>
-#include <boost/python/args.hpp>
 
 using namespace dlib;
-using namespace std;
-using namespace boost::python;
+namespace py = pybind11;
 
 typedef matrix<double,0,1> dense_vect; 
 typedef std::vector<std::pair<unsigned long,double> > sparse_vect;
@@ -279,9 +277,9 @@ struct segmenter_params
 };
 
 
-string segmenter_params__str__(const segmenter_params& p)
+std::string segmenter_params__str__(const segmenter_params& p)
 {
-    ostringstream sout;
+    std::ostringstream sout;
     if (p.use_BIO_model)
         sout << "BIO,";
     else
@@ -309,9 +307,9 @@ string segmenter_params__str__(const segmenter_params& p)
     return trim(sout.str());
 }
 
-string segmenter_params__repr__(const segmenter_params& p)
+std::string segmenter_params__repr__(const segmenter_params& p)
 {
-    ostringstream sout;
+    std::ostringstream sout;
     sout << "<";
     sout << segmenter_params__str__(p);
     sout << ">";
@@ -776,11 +774,12 @@ const segmenter_test cross_validate_sequence_segmenter2 (
 
 // ----------------------------------------------------------------------------------------
 
-void bind_sequence_segmenter()
+void bind_sequence_segmenter(py::module& m)
 {
-    class_<segmenter_params>("segmenter_params",
+    py::class_<segmenter_params>(m, "segmenter_params",
 "This class is used to define all the optional parameters to the    \n\
 train_sequence_segmenter() and cross_validate_sequence_segmenter() routines.   ")
+        .def(py::init<>())
         .def_readwrite("use_BIO_model", &segmenter_params::use_BIO_model)
         .def_readwrite("use_high_order_features", &segmenter_params::use_high_order_features)
         .def_readwrite("allow_negative_weights", &segmenter_params::allow_negative_weights)
@@ -792,36 +791,35 @@ train_sequence_segmenter() and cross_validate_sequence_segmenter() routines.   "
         .def_readwrite("be_verbose", &segmenter_params::be_verbose)
         .def("__repr__",&segmenter_params__repr__)
         .def("__str__",&segmenter_params__str__)
-        .def_pickle(serialize_pickle<segmenter_params>());
+        .def(py::pickle(&getstate<segmenter_params>, &setstate<segmenter_params>));
 
-    class_<segmenter_type> ("segmenter_type", "This object represents a sequence segmenter and is the type of object "
+    py::class_<segmenter_type> (m, "segmenter_type", "This object represents a sequence segmenter and is the type of object "
         "returned by the dlib.train_sequence_segmenter() routine.")
         .def("__call__", &segmenter_type::segment_sequence_dense)
         .def("__call__", &segmenter_type::segment_sequence_sparse)
-        .def_readonly("weights", &segmenter_type::get_weights)
-        .def_pickle(serialize_pickle<segmenter_type>());
+        .def_property_readonly("weights", &segmenter_type::get_weights)
+        .def(py::pickle(&getstate<segmenter_type>, &setstate<segmenter_type>));
 
-    class_<segmenter_test> ("segmenter_test", "This object is the output of the dlib.test_sequence_segmenter() and "
+    py::class_<segmenter_test> (m, "segmenter_test", "This object is the output of the dlib.test_sequence_segmenter() and "
         "dlib.cross_validate_sequence_segmenter() routines.")
         .def_readwrite("precision", &segmenter_test::precision)
         .def_readwrite("recall", &segmenter_test::recall)
         .def_readwrite("f1", &segmenter_test::f1)
         .def("__repr__",&segmenter_test__repr__)
         .def("__str__",&segmenter_test__str__)
-        .def_pickle(serialize_pickle<segmenter_test>());
+        .def(py::pickle(&getstate<segmenter_test>, &setstate<segmenter_test>));
 
-    using boost::python::arg;
-    def("train_sequence_segmenter", train_dense, (arg("samples"), arg("segments"), arg("params")=segmenter_params()));
-    def("train_sequence_segmenter", train_sparse, (arg("samples"), arg("segments"), arg("params")=segmenter_params()));
+    m.def("train_sequence_segmenter", train_dense, py::arg("samples"), py::arg("segments"), py::arg("params")=segmenter_params());
+    m.def("train_sequence_segmenter", train_sparse, py::arg("samples"), py::arg("segments"), py::arg("params")=segmenter_params());
 
 
-    def("test_sequence_segmenter", test_sequence_segmenter1);
-    def("test_sequence_segmenter", test_sequence_segmenter2);
+    m.def("test_sequence_segmenter", test_sequence_segmenter1);
+    m.def("test_sequence_segmenter", test_sequence_segmenter2);
 
-    def("cross_validate_sequence_segmenter", cross_validate_sequence_segmenter1,
-        (arg("samples"), arg("segments"), arg("folds"), arg("params")=segmenter_params()));
-    def("cross_validate_sequence_segmenter", cross_validate_sequence_segmenter2,
-        (arg("samples"), arg("segments"), arg("folds"), arg("params")=segmenter_params()));
+    m.def("cross_validate_sequence_segmenter", cross_validate_sequence_segmenter1,
+        py::arg("samples"), py::arg("segments"), py::arg("folds"), py::arg("params")=segmenter_params());
+    m.def("cross_validate_sequence_segmenter", cross_validate_sequence_segmenter2,
+        py::arg("samples"), py::arg("segments"), py::arg("folds"), py::arg("params")=segmenter_params());
 }
 
 
